@@ -12,6 +12,9 @@
 #include "filter_char.hpp"
 #include "posib_err.hpp"
 #include "vector.hpp"
+#include "string_pair_enumeration.hpp"
+
+#include <stdio.h>
 
 namespace acommon {
 
@@ -19,10 +22,12 @@ namespace acommon {
   class Speller;
   class IndividualFilter;
   class StringList;
+  struct ConfigModule;
 
   class Filter : public CanHaveError {
   public:
-    bool empty() const {return filters_.empty();} 
+    
+    bool empty() const {return filters_.empty();}
     void clear();
     void reset();
     void process(FilterChar * & start, FilterChar * & stop);
@@ -31,19 +36,32 @@ namespace acommon {
     // filters to use.
     Filter();
     ~Filter();
-  private:
+
+ private:
     typedef Vector<IndividualFilter *> Filters;
     Filters filters_;
   };
 
-  void set_mode_from_extension(Config * config,
-			       ParmString filename);
+  PosibErr<void> set_mode_from_extension(Config * config,
+                                         ParmString filename,
+                                         FILE * in = NULL);
   
   PosibErr<void> setup_filter(Filter &, Config *, 
 			      bool use_decoder, 
 			      bool use_filter, 
 			      bool use_encoder);
+  void activate_dynamic_filteroptions(Config *c);
+  void activate_filter_modes(Config * config);
 
-}
+  void load_all_filters(Config * config);
+
+  PosibErr<bool> verify_version(const char * relOp, const char * actual,
+                                const char * required);
+  PosibErr<void> check_version(char * requirement);
+
+  PosibErr<StringPairEnumeration *> available_filters(Config *);
+  PosibErr<StringPairEnumeration *> available_filter_modes(Config *);
+
+};
 
 #endif /* ASPELL_FILTER__HPP */
