@@ -9,7 +9,7 @@
 
 #include "fstream.hpp"
 
-#if defined(__CYGWIN__) || defined (_WIN32)
+#if defined(__CYGWIN__)
 #define DLLIMPORT __declspec(dllimport)
 #else
 #define DLLIMPORT
@@ -19,7 +19,7 @@ namespace acommon {
 
   // These streams for the time being will be based on stdin, stdout,
   // and stderr respectfully.  So it is safe to use the standard C
-  // functions.  It is also safe to assume that modifcations to the
+  // functions.  It is also safe to assume that modifications to the
   // state of the standard streams will effect these.
 
   extern DLLIMPORT FStream CIN;
