@@ -20,11 +20,7 @@
 
 namespace acommon {
 
-#ifndef __GNUC__
-#define __attribute__(x)
-#endif
-
-#define LOCK(l) const Lock __attribute__((__unused__)) the_lock(l) 
+#define LOCK(l) const Lock the_lock(l);
 
 #ifdef USE_POSIX_MUTEX
   class Mutex {
@@ -60,6 +56,6 @@ namespace acommon {
     Lock(Mutex * l) : lock_(l) {if (lock_) lock_->lock();}
     ~Lock() {if (lock_) lock_->unlock();}
   };
-}
+};
 
 #endif
