@@ -4,6 +4,17 @@
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
 
+#include <stdarg.h>
+#include <stdio.h>
+
+#ifndef va_copy
+#  ifdef __va_copy
+#    define va_copy __va_copy
+#  else
+#    define va_copy(dst, src) memcpy(&dst, &src, sizeof(va_list))
+#  endif
+#endif
+
 #include "string.hpp"
 #include "asc_ctype.hpp"
 
@@ -31,7 +42,7 @@ namespace acommon {
     int res;
     va_list ap;
   loop: {
-      size_t avail = storage_end_ - end_;
+      int avail = storage_end_ - end_;
       if (avail > 1024) return -1;
       va_copy(ap,ap0);
       res = vsnprintf(end_, avail, format, ap);
@@ -39,10 +50,10 @@ namespace acommon {
       if (res < 0) {
         reserve_i(); goto loop;
       } else if (res > avail) {
-        reserve_i(size() + res - 1); goto loop;
+        reserve_i(size() + res); goto loop;
       }
     }
-    end_ += res - 1;
+    end_ += res;
     return res;
   }
   
@@ -68,5 +79,4 @@ namespace acommon {
     }
     return size == 0;
   }
-
 }
