@@ -15,17 +15,18 @@
 #include "errors.hpp"
 #include "convert.hpp"
 
-#define FILTER_PROGRESS_CONTROL "tex-filter-debug.log"
-#include "loadable-filter-API.hpp"
-#include <stdio.h>
+//#define FILTER_PROGRESS_CONTROL "tex-filter-debug.log"
+//#include "filter_debug.hpp"
+//#include <stdio.h>
 #include <cstdio>
 #include "filter_char_vector.hpp"
 #include "string_list.hpp"
 #include "string_enumeration.hpp"
 
 
-namespace acommon {
+namespace {
 
+  using namespace acommon;
 
   class TexFilter : public IndividualFilter 
   {
@@ -45,8 +46,8 @@ namespace acommon {
 
     class Commands : public StringMap {
     public:
-      PosibErr<bool> add(ParmString to_add);
-      PosibErr<bool> remove(ParmString to_rem);
+      PosibErr<bool> add(ParmStr to_add);
+      PosibErr<bool> remove(ParmStr to_rem);
     };
     
     Commands commands;
@@ -90,9 +91,9 @@ namespace acommon {
     //fprintf(stderr,"name %s \n",name_);
 
     commands.clear();
-    opts->retrieve_list("filter-tex-command", &commands);
+    opts->retrieve_list("f-tex-command", &commands);
     
-    check_comments = opts->retrieve_bool("filter-tex-check-comments");
+    check_comments = opts->retrieve_bool("f-tex-check-comments");
 
     reset();
     return true;
@@ -241,17 +242,19 @@ namespace acommon {
   // TexFilter::Commands
   //
 
-  PosibErr<bool> TexFilter::Commands::add(ParmString value) {
+  PosibErr<bool> TexFilter::Commands::add(ParmStr value) {
     int p1 = 0;
     while (!asc_isspace(value[p1])) {
       if (value[p1] == '\0') 
-	return make_err(bad_value, value,"","a string of o,O,p, or P");
+	return make_err(bad_value, value,"",
+                        _("a string of 'o','O','p',or 'P'"));
       ++p1;
     }
     int p2 = p1 + 1;
     while (asc_isspace(value[p2])) {
       if (value[p2] == '\0') 
-	return make_err(bad_value, value,"","a string of o,O,p, or P");
+	return make_err(bad_value, value,"",
+                        _("a string of 'o','O','p',or 'P'"));
       ++p2;
     }
     String t1; t1.assign(value,p1);
@@ -259,13 +262,13 @@ namespace acommon {
     return StringMap::replace(t1, t2);
   }
   
-  PosibErr<bool> TexFilter::Commands::remove(ParmString value) {
+  PosibErr<bool> TexFilter::Commands::remove(ParmStr value) {
     int p1 = 0;
     while (!asc_isspace(value[p1]) && value[p1] != '\0') ++p1;
     String temp; temp.assign(value,p1);
     return StringMap::remove(temp);
   }
-  
+#if 0  
   //
   //
   class Recode {
@@ -352,7 +355,7 @@ namespace acommon {
       keylen=strlen(code[0]);
     }
     while (begin && end && *begin) {
-      while (end && *end && (*end != ':')) {
+      while (end && *end && (*end != ',')) {
         end++;
       }
       if (end  && (begin != end)) {
@@ -478,16 +481,16 @@ namespace acommon {
 
   PosibErr<bool> TexEncoder::setup(Config * config) {
     name_ = "tex-encoder";
-    order_num_ = 0.4;
+    order_num_ = 0.40;
 
     StringList multibytechars;
 
-    config->retrieve_list("filter-tex-multi-byte", &multibytechars);
+    config->retrieve_list("f-tex-multi-byte", &multibytechars);
 
     Conv conv; // this a quick and dirty fix witch will only work for
                // iso-8859-1.  Full unicode support needs to be
                // implemented
-    conv.setup(*config, "utf-8", "iso-8859-1");
+    conv.setup(*config, "utf-8", "iso-8859-1", NormNone);
 
     StringEnumeration * multibytelist=multibytechars.elements();
     const char * multibyte0=NULL;
@@ -584,12 +587,12 @@ namespace acommon {
 
     StringList multibytechars;
     
-    config->retrieve_list("filter-tex-multi-byte", &multibytechars);
+    config->retrieve_list("f-tex-multi-byte", &multibytechars);
 
     Conv conv; // this a quick and dirty fix witch will only work for
                // iso-8859-1.  Full unicode support needs to be
                // implemented
-    conv.setup(*config, "utf-8", "iso-8859-1");
+    conv.setup(*config, "utf-8", "iso-8859-1", NormNone);
 
     StringEnumeration * multibytelist=multibytechars.elements();
     const char * multibyte0=NULL;
@@ -606,6 +609,9 @@ namespace acommon {
                 name_,multibyte);
         continue;
       }
+      FDEBUGPRINTF("next multibyte chars `");
+      FDEBUGPRINTF(multibyte);
+      FDEBUGPRINTF("'\n");
 
       int countmulti=0;
 
@@ -613,6 +619,9 @@ namespace acommon {
              !multibytes[countmulti].setExpansion(multibyte)) {
         countmulti++;
       }
+      FDEBUGPRINTF("next multibyte chars `");
+      FDEBUGPRINTF(multibyte);
+      FDEBUGPRINTF("'\n");
       if (countmulti >= multibytes.size()) {
         multibytes.resize(multibytes.size()+1);
         if (!multibytes[multibytes.size()-1].setExpansion(multibyte)) {
@@ -620,10 +629,13 @@ namespace acommon {
           continue;
         }
       }
+      FDEBUGPRINTF("next multibyte chars `");
+      FDEBUGPRINTF(multibyte);
+      FDEBUGPRINTF("'\n");
     }
     StringList hyphenChars;
     
-    config->retrieve_list("filter-tex-hyphen", &hyphenChars);
+    config->retrieve_list("f-tex-hyphen", &hyphenChars);
 
     StringEnumeration * hyphenList=hyphenChars.elements();
     const char * hyphen=NULL;
@@ -736,8 +748,15 @@ namespace acommon {
   TexDecoder::~TexDecoder(){
     FDEBUGCLOSE;
   }
-    
-  ACTIVATE_ENCODER(acommon,TexEncoder,tex);
-  ACTIVATE_FILTER(acommon,TexFilter,tex);
-  ACTIVATE_DECODER(acommon,TexDecoder,tex);
+#endif
 }
+
+C_EXPORT 
+IndividualFilter * new_aspell_tex_filter() {return new TexFilter;}
+
+#if 0
+C_EXPORT 
+IndividualFilter * new_aspell_tex_decoder() {return new TexDecoder;}
+C_EXPORT 
+IndividualFilter * new_aspell_tex_encoder() {return new TexEncoder;}
+#endif
