@@ -14,6 +14,7 @@
 #include "language.hpp"
 #include "speller_impl.hpp"
 #include "cache-t.hpp"
+#include "vararray.hpp"
 
 #include "iostream.hpp"
 
@@ -68,7 +69,8 @@ namespace aspeller {
   Dictionary::Dictionary(BasicType t, const char * n)
     : Cacheable(&dict_cache), lang_(), id_(), 
       basic_type(t), class_name(n),
-      affix_compressed(false), have_soundslike(false), 
+      affix_compressed(false), 
+      invisible_soundslike(false), soundslike_root_only(false),
       fast_scan(false), fast_lookup(false)
   {
     id_.reset(new Id(this));
@@ -88,12 +90,13 @@ namespace aspeller {
     return no_err;
   }
  
-  PosibErr<void> Dictionary::set_check_lang (ParmString l, const Config * config)
+  PosibErr<void> Dictionary::set_check_lang (ParmString l, Config & config)
   {
     if (lang_ == 0) {
-      PosibErr<Language *> res = new_language(*config, l);
+      PosibErr<Language *> res = new_language(config, l);
       if (res.has_err()) return res;
       lang_.reset(res.data);
+      lang_->set_lang_defaults(config);
       set_lang_hook(config);
     } else {
       if (l != lang_->name())
@@ -175,21 +178,32 @@ namespace aspeller {
 
   PosibErr<void> Dictionary::add_repl(ParmString mis, ParmString cor) 
   {
-    return add_repl(mis, cor, have_soundslike ? lang()->to_soundslike(mis) : "");
+    if (!invisible_soundslike) {
+      VARARRAY(char, sl, mis.size() + 1);
+      lang()->LangImpl::to_soundslike(sl, mis.str(), mis.size());
+      return add_repl(mis, cor, sl);
+    } else {
+      return add_repl(mis, cor, "");
+    }
   }
 
   PosibErr<void> Dictionary::add(ParmString w) 
   {
-    return add(w, have_soundslike ? lang()->to_soundslike(w) : "");
+    if (!invisible_soundslike) {
+      VARARRAY(char, sl, w.size() + 1);
+      lang()->LangImpl::to_soundslike(sl, w.str(), w.size());
+      return add(w, sl);
+    } else {
+      return add(w, "");
+    }
   }
 
   //
   // Default implementation;
   //
 
-  PosibErr<void> Dictionary::load(ParmString, const Config &, 
-                                  LocalDictList *, 
-                                  SpellerImpl *, const LocalDictInfo *) 
+  PosibErr<void> Dictionary::load(ParmString, Config &, 
+                                  DictList *, SpellerImpl *) 
   {
     return make_err(unimplemented_method, "load", class_name);
   }
@@ -240,13 +254,13 @@ namespace aspeller {
     return false;
   }
   
-  bool Dictionary::lookup (ParmString word, WordEntry &,
-                           const SensitiveCompare &) const
+  bool Dictionary::lookup (ParmString word, const SensitiveCompare *,
+                           WordEntry &) const
   {
     return false;
   }
   
-  bool Dictionary::stripped_lookup(ParmString, WordEntry &) const 
+  bool Dictionary::clean_lookup(ParmString, WordEntry &) const 
   {
     return false;
   }
@@ -296,7 +310,7 @@ namespace aspeller {
     return make_err(unimplemented_method, "remove_repl", class_name);
   }
   
-  Enumeration<const LocalDict *> * Dictionary::dictionaries() const 
+  DictsEnumeration * Dictionary::dictionaries() const 
   {
     return 0;
   }
@@ -308,13 +322,11 @@ namespace aspeller {
 
   OStream & WordEntry::write (OStream & o,
                               const Language & l,
-                              const ConvertWord & cw,
                               Convert * c) const
   {
     String w;
     CharVector buf;
-    cw.convert(word, w);
-    write_conv(w);
+    write_conv(word);
     if (aff && *aff) {
       o << '/';
       write_conv(aff);
@@ -322,15 +334,14 @@ namespace aspeller {
     return o;
   }
 
-  PosibErr<void> add_data_set(ParmString fn,
-                              const Config & config,
-                              LocalDict & res,
-                              LocalDictList * new_dicts,
-                              SpellerImpl * speller,
-                              const LocalDictInfo * local_info,
-                              ParmString dir,
-                              DataType allowed)
+  PosibErr<Dict *> add_data_set(ParmString fn,
+                                Config & config,
+                                DictList * new_dicts,
+                                SpellerImpl * speller,
+                                ParmString dir,
+                                DataType allowed)
   {
+    Dict * res = 0;
     static const char * suffix_list[] = {"", ".multi", ".alias", 
 					 ".spcl", ".special",
 					 ".pws", ".prepl"};
@@ -394,20 +405,19 @@ namespace aspeller {
     Dict::Id id(0,Dict::FileName(true_file_name));
 
     if (speller != 0) {
-      const LocalDict * d = speller->locate(id);
+      const SpellerDict * d = speller->locate(id);
       if (d != 0) {
-        res = *d;
-        return no_err;
+        return d->dict;
       }
     }
 
-    res.dict = 0;
+    res = 0;
 
     if (actual_type == DT_ReadOnly) { // try to get it from the cache
-      res.dict = dict_cache.find(id);
+      res = dict_cache.find(id);
     }
 
-    if (!res.dict) {
+    if (!res) {
 
       StackPtr<Dict> w;
       switch (actual_type) {
@@ -427,52 +437,24 @@ namespace aspeller {
         abort();
       }
 
-      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller, local_info));
+      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller));
 
       if (actual_type == DT_ReadOnly)
         dict_cache.add(w);
       
-      res.dict = w.release();
+      res = w.release();
 
     } else {
       
-      res.dict->copy();
+      res->copy();
       
     }
 
-    if (local_info) {
-      res.set(*local_info);
-      res.set_language(res.dict->lang());
-    } else
-      res.set(res.dict->lang(), config);
     if (new_dicts)
       new_dicts->add(res);
     
-    return no_err;
+    return res;
   }
 
-  //
-  // LocalDictInfo
-  //
-  
-  void LocalDictInfo::set_language(const Language * l)
-  {
-    compare.lang = l;
-    convert.lang = l;
-  }
-
-  void LocalDictInfo::set(const Language * l, const Config & c, bool strip)
-  {
-    if (c.have("strip-accents"))
-      strip = c.retrieve_bool("strip-accents");
-
-    compare.lang = l;
-    compare.case_insensitive = c.retrieve_bool("ignore-case");
-    compare.ignore_accents   = c.retrieve_bool("ignore-accents");
-    compare.strip_accents    = strip;
-    convert.lang = l;
-    convert.strip_accents = strip;
-  }
-  
 }
 
