@@ -64,14 +64,14 @@ namespace aspeller {
   }
 
   PosibErr<void> SpellerImpl::store_replacement(MutableString mis, 
-						MutableString cor)
+                                                MutableString cor)
   {
     return SpellerImpl::store_replacement(mis,cor,true);
   }
 
   PosibErr<void> SpellerImpl::store_replacement(const String & mis, 
-						const String & cor, 
-						bool memory) 
+                                                const String & cor, 
+                                                bool memory) 
   {
     if (ignore_repl) return no_err;
     if (!repl_) return no_err;
@@ -80,6 +80,7 @@ namespace aspeller {
     const char * first_word = sugels->next();
     CheckInfo w1, w2;
     String cor1, cor2;
+    String buf;
     bool correct = false;
     if (pos = cor.find(' '), pos == String::npos) {
       cor1 = cor;
@@ -92,22 +93,23 @@ namespace aspeller {
     if (correct) {
       String cor_orignal_casing(cor1);
       if (!cor2.empty()) {
- 	cor_orignal_casing += cor[pos];
- 	cor_orignal_casing += cor2;
+        cor_orignal_casing += cor[pos];
+        cor_orignal_casing += cor2;
       }
       if (first_word == 0 || cor != first_word) {
- 	repl_->add_repl(aspeller::to_lower(lang(), mis), cor_orignal_casing);
+        lang().to_lower(buf, mis.str());
+        repl_->add_repl(buf, cor_orignal_casing);
       }
       
       if (memory && prev_cor_repl_ == mis) 
- 	store_replacement(prev_mis_repl_, cor, false);
+        store_replacement(prev_mis_repl_, cor, false);
       
     } else { //!correct
       
       if (memory) {
-	 if (prev_cor_repl_ != mis)
- 	  prev_mis_repl_ = mis;
- 	prev_cor_repl_ = cor;
+         if (prev_cor_repl_ != mis)
+          prev_mis_repl_ = mis;
+        prev_cor_repl_ = cor;
        }
     }
     return no_err;
@@ -131,7 +133,7 @@ namespace aspeller {
     WS::const_iterator i   = check_ws.begin();
     WS::const_iterator end = check_ws.end();
     do {
-      if (i->dict->lookup(w, w0, i->compare)) return true;
+      if ((*i)->lookup(w, &s_cmp, w0)) return true;
       ++i;
     } while (i != end);
     return false;
@@ -152,35 +154,54 @@ namespace aspeller {
     return false;
   }
 
+  inline bool SpellerImpl::check2(char * word, /* it WILL modify word */
+                                  bool try_uppercase,
+                                  CheckInfo & ci, GuessInfo * gi)
+  {
+    bool res = check_affix(word, ci, gi);
+    if (res) return true;
+    if (!try_uppercase) return false;
+    char t = *word;
+    *word = lang_->to_title(t);
+    res = check_affix(word, ci, gi);
+    *word = t;
+    if (res) return true;
+    return false;
+  }
+
   PosibErr<bool> SpellerImpl::check(char * word, char * word_end, 
                                     /* it WILL modify word */
-				    unsigned int run_together_limit,
-				    CheckInfo * ci, GuessInfo * gi)
+                                    bool try_uppercase,
+                                    unsigned run_together_limit,
+                                    CheckInfo * ci, GuessInfo * gi)
   {
     assert(run_together_limit <= 8); // otherwise it will go above the 
                                      // bounds of the word array
     clear_check_info(*ci);
-    bool res = check_affix(word, *ci, gi);
+    bool res = check2(word, try_uppercase, *ci, gi);
     if (res) return true;
     if (run_together_limit <= 1) return false;
-    for (char * i = word + run_together_start_len_; 
-	 i <= word_end - run_together_start_len_;
-	 ++i) 
-      {
-	char t = *i;
-	*i = '\0';
-        //FIXME: clear ci, gi?
-	res = check_affix(word, *ci, gi);
-	*i = t;
-	if (!res) continue;
-	if (check(i, word_end, run_together_limit - 1, ci + 1, 0)) {
-          ci->next = ci + 1;
-	  return true;
-        }
+    enum {Yes, No, Unknown} is_title = try_uppercase ? Yes : Unknown;
+    for (char * i = word + run_together_min_; 
+         i <= word_end - run_together_min_;
+         ++i) 
+    {
+      char t = *i;
+      *i = '\0';
+      //FIXME: clear ci, gi?
+      res = check2(word, try_uppercase, *ci, gi);
+      if (!res) {*i = t; continue;}
+      if (is_title == Unknown)
+        is_title = lang_->case_pattern(word) == FirstUpper ? Yes : No;
+      *i = t;
+      if (check(i, word_end, is_title == Yes, run_together_limit - 1, ci + 1, 0)) {
+        ci->compound = true;
+        ci->next = ci + 1;
+        return true;
       }
+    }
     return false;
   }
-  
 
   //////////////////////////////////////////////////////////////////////
   //
@@ -191,7 +212,7 @@ namespace aspeller {
     SpellerDict * i = dicts_;
     for (; i; i = i->next) {
       if  (i->save_on_saveall)
-	RET_ON_ERR(i->dict->synchronize());
+        RET_ON_ERR(i->dict->synchronize());
     }
     return no_err;
   }
@@ -224,16 +245,18 @@ namespace aspeller {
     return 0;
   }
 
-  void SpellerImpl::add_dict(SpellerDict * wc)
+  PosibErr<void> SpellerImpl::add_dict(SpellerDict * wc)
   {
     Dict * w = wc->dict;
     assert(locate(w->id()) == 0);
 
-    if (!lang_) 
-    {
+    if (!lang_) {
       lang_.copy(w->lang());
       config_->replace("lang", lang_name());
       config_->replace("language-tag", lang_name());
+    } else {
+      if (strcmp(lang_->name(), w->lang()->name()) != 0)
+        return make_err(mismatched_language, lang_->name(), w->lang()->name());
     }
 
     // add to master list
@@ -261,6 +284,8 @@ namespace aspeller {
     case none_id:
       break;
     }
+
+    return no_err;
   }
 
   //////////////////////////////////////////////////////////////////////
@@ -284,18 +309,18 @@ namespace aspeller {
       Fun(WithInt  m) : with_int (m) {}
       Fun(WithBool m) : with_bool(m) {}
       PosibErr<void> call(SpellerImpl * m, const char * val) const 
-	{return (*with_str) (m,val);}
+        {return (*with_str) (m,val);}
       PosibErr<void> call(SpellerImpl * m, int val)          const 
-	{return (*with_int) (m,val);}
+        {return (*with_int) (m,val);}
       PosibErr<void> call(SpellerImpl * m, bool val)         const 
-	{return (*with_bool)(m,val);}
+        {return (*with_bool)(m,val);}
     } fun;
     typedef SpellerImpl::ConfigNotifier CN;
   };
 
   template <typename T>
   PosibErr<void> callback(SpellerImpl * m, const KeyInfo * ki, T value, 
-			  UpdateMember::Type t);
+                          UpdateMember::Type t);
   
   class SpellerImpl::ConfigNotifier : public Notifier {
   private:
@@ -311,7 +336,7 @@ namespace aspeller {
     PosibErr<void> item_updated(const KeyInfo * ki, bool value) {
       return callback(speller_, ki, value, UpdateMember::Bool);
     }
-    PosibErr<void> item_updated(const KeyInfo * ki, ParmString value) {
+    PosibErr<void> item_updated(const KeyInfo * ki, ParmStr value) {
       return callback(speller_, ki, value, UpdateMember::String);
     }
 
@@ -320,12 +345,17 @@ namespace aspeller {
       return no_err;
     }
     static PosibErr<void> ignore_accents(SpellerImpl * m, bool value) {
-      abort(); return no_err;
+      return no_err;
     }
     static PosibErr<void> ignore_case(SpellerImpl * m, bool value) {
-      abort(); return no_err;
+      m->s_cmp.case_insensitive = value;
+      m->s_cmp_begin.case_insensitive = value;
+      m->s_cmp_middle.case_insensitive = value;
+      m->s_cmp_end.case_insensitive = value;
+      return no_err;
     }
     static PosibErr<void> ignore_repl(SpellerImpl * m, bool value) {
+      
       m->ignore_repl = value;
       return no_err;
     }
@@ -341,22 +371,20 @@ namespace aspeller {
     }
     static PosibErr<void> run_together(SpellerImpl * m, bool value) {
       m->unconditional_run_together_ = value;
+      m->run_together = m->unconditional_run_together_;
       return no_err;
     }
     static PosibErr<void> run_together_limit(SpellerImpl * m, int value) {
       if (value > 8) {
-	m->config()->replace("run-together-limit", "8");
-	// will loop back
+        m->config()->replace("run-together-limit", "8");
+        // will loop back
       } else {
-	m->run_together_limit_ = value;
+        m->run_together_limit_ = value;
       }
       return no_err;
     }
     static PosibErr<void> run_together_min(SpellerImpl * m, int value) {
       m->run_together_min_ = value;
-      if (m->unconditional_run_together_ 
-	  && m->run_together_min_ < m->run_together_start_len_)
-	m->run_together_start_len_ = m->run_together_min_;
       return no_err;
     }
     
@@ -368,22 +396,22 @@ namespace aspeller {
     ,{"ignore-accents",UpdateMember::Bool,    UpdateMember::CN::ignore_accents}
     ,{"ignore-case",   UpdateMember::Bool,    UpdateMember::CN::ignore_case}
     ,{"ignore-repl",   UpdateMember::Bool,    UpdateMember::CN::ignore_repl}
-    ,{"save-repl",     UpdateMember::Bool,    UpdateMember::CN::save_repl}
+    //,{"save-repl",     UpdateMember::Bool,    UpdateMember::CN::save_repl}
     ,{"sug-mode",      UpdateMember::String,  UpdateMember::CN::sug_mode}
     ,{"run-together",  
-	UpdateMember::Bool,    
-	UpdateMember::CN::run_together}
+        UpdateMember::Bool,    
+        UpdateMember::CN::run_together}
     ,{"run-together-limit",  
-	UpdateMember::Int,    
-	UpdateMember::CN::run_together_limit}
+        UpdateMember::Int,    
+        UpdateMember::CN::run_together_limit}
     ,{"run-together-min",  
-	UpdateMember::Int,    
-	UpdateMember::CN::run_together_min}
+        UpdateMember::Int,    
+        UpdateMember::CN::run_together_min}
   };
 
   template <typename T>
   PosibErr<void> callback(SpellerImpl * m, const KeyInfo * ki, T value, 
-			  UpdateMember::Type t) 
+                          UpdateMember::Type t) 
   {
     const UpdateMember * i
       = update_members;
@@ -391,10 +419,10 @@ namespace aspeller {
       = i + sizeof(update_members)/sizeof(UpdateMember);
     while (i != end) {
       if (strcmp(ki->name, i->name) == 0) {
-	if (i->type == t) {
-	  RET_ON_ERR(i->fun.call(m, value));
-	  break;
-	}
+        if (i->type == t) {
+          RET_ON_ERR(i->fun.call(m, value));
+          break;
+        }
       }
       ++i;
     }
@@ -408,18 +436,18 @@ namespace aspeller {
 
   SpellerImpl::SpellerImpl() 
     : Speller(0) /* FIXME */, ignore_repl(true), 
-      dicts_(0), personal_(0), session_(0), repl_(0), main_(0),
-      guess_info(7)
+      dicts_(0), personal_(0), session_(0), repl_(0), main_(0)
   {}
 
-  inline void add_dicts(SpellerImpl * sp, LocalDictList & d)
+  inline PosibErr<void> add_dicts(SpellerImpl * sp, DictList & d)
   {
     for (;!d.empty(); d.pop())
     {
-      if (!sp->locate(d.last().dict->id())) {
-        sp->add_dict(new SpellerDict(d.last()));
+      if (!sp->locate(d.last()->id())) {
+        RET_ON_ERR(sp->add_dict(new SpellerDict(d.last())));
       }
     }
+    return no_err;
   }
 
   PosibErr<void> SpellerImpl::setup(Config * c) {
@@ -429,26 +457,33 @@ namespace aspeller {
     ignore_repl = config_->retrieve_bool("ignore-repl");
     ignore_count = config_->retrieve_int("ignore");
 
-    LocalDict res;
-    LocalDictList to_add;
-    RET_ON_ERR(add_data_set(config_->retrieve("master-path"), *config_, res, &to_add, this));
-    add_dicts(this, to_add);
+    DictList to_add;
+    RET_ON_ERR(add_data_set(config_->retrieve("master-path"), *config_, &to_add, this));
+    RET_ON_ERR(add_dicts(this, to_add));
 
-    use_soundslike = true;
+    s_cmp.lang = lang_;
+    s_cmp.case_insensitive = config_->retrieve_bool("ignore-case");
 
-    for (SpellerDict * i = dicts_; i; i = i->next) {
-      
-      if (i->dict->basic_type == Dict::basic_dict)
-        use_soundslike = use_soundslike && i->dict->have_soundslike;
-    }
+    s_cmp_begin.lang = lang_; 
+    s_cmp_begin.case_insensitive = s_cmp.case_insensitive;
+    s_cmp_begin.end = false;
+
+    s_cmp_middle.lang = lang_;
+    s_cmp_middle.case_insensitive = s_cmp.case_insensitive;
+    s_cmp_middle.begin = false;
+    s_cmp_middle.end   = false;
+
+    s_cmp_end.lang = lang_;
+    s_cmp_end.case_insensitive = s_cmp.case_insensitive;
+    s_cmp_end.begin = false;
 
     StringList extra_dicts;
     config_->retrieve_list("extra-dicts", &extra_dicts);
     StringListEnumeration els = extra_dicts.elements_obj();
     const char * dict_name;
     while ( (dict_name = els.next()) != 0) {
-      RET_ON_ERR(add_data_set(dict_name,*config_, res, &to_add, this));
-      add_dicts(this, to_add);
+      RET_ON_ERR(add_data_set(dict_name,*config_, &to_add, this));
+      RET_ON_ERR(add_dicts(this, to_add));
     }
 
     bool use_other_dicts = config_->retrieve_bool("use-other-dicts");
@@ -457,51 +492,51 @@ namespace aspeller {
     {
       Dictionary * temp;
       temp = new_default_writable_dict();
-      temp->have_soundslike = use_soundslike;
       PosibErrBase pe = temp->load(config_->retrieve("personal-path"),*config_);
       if (pe.has_err(cant_read_file))
-	temp->set_check_lang(lang_name(), config_);
+        temp->set_check_lang(lang_name(), *config_);
       else if (pe.has_err())
-	return pe;
-      add_dict(new SpellerDict(temp, lang_, *config_, personal_id));
+        return pe;
+      RET_ON_ERR(add_dict(new SpellerDict(temp, *config_, personal_id)));
     }
     
     if (use_other_dicts && !session_)
     {
       Dictionary * temp;
       temp = new_default_writable_dict();
-      temp->have_soundslike = use_soundslike;
-      temp->set_check_lang(lang_name(), config_);
-      add_dict(new SpellerDict(temp, lang_, *config_, session_id));
+      temp->set_check_lang(lang_name(), *config_);
+      RET_ON_ERR(add_dict(new SpellerDict(temp, *config_, session_id)));
     }
      
     if (use_other_dicts && !repl_)
     {
       ReplacementDict * temp = new_default_replacement_dict();
-      temp->have_soundslike = use_soundslike;
       PosibErrBase pe = temp->load(config_->retrieve("repl-path"),*config_);
       if (pe.has_err(cant_read_file))
-	temp->set_check_lang(lang_name(), config_);
+        temp->set_check_lang(lang_name(), *config_);
       else if (pe.has_err())
-	return pe;
-      add_dict(new SpellerDict(temp, lang_, *config_, personal_repl_id));
+        return pe;
+      RET_ON_ERR(add_dict(new SpellerDict(temp, *config_, personal_repl_id)));
     }
 
-    const char * sys_enc = lang_->charset();
-    if (!config_->have("encoding"))
-      config_->replace("encoding", sys_enc);
+    const char * sys_enc = lang_->charmap();
     String user_enc = config_->retrieve("encoding");
+    if (user_enc == "none") {
+      config_->replace("encoding", sys_enc);
+      user_enc = sys_enc;
+    }
 
     PosibErr<Convert *> conv;
-    conv = new_convert(*c, user_enc, sys_enc);
+    conv = new_convert(*c, user_enc, sys_enc, NormFrom);
     if (conv.has_err()) return conv;
     to_internal_.reset(conv);
-    conv = new_convert(*c, sys_enc, user_enc);
+    conv = new_convert(*c, sys_enc, user_enc, NormTo);
     if (conv.has_err()) return conv;
     from_internal_.reset(conv);
 
     unconditional_run_together_ = config_->retrieve_bool("run-together");
-
+    run_together = unconditional_run_together_;
+    
     run_together_limit_  = config_->retrieve_int("run-together-limit");
     if (run_together_limit_ > 8) {
       config_->replace("run-together-limit", "8");
@@ -509,10 +544,6 @@ namespace aspeller {
     }
     run_together_min_    = config_->retrieve_int("run-together-min");
 
-    if (unconditional_run_together_ 
-	&& run_together_min_ < run_together_start_len_)
-      run_together_start_len_ = run_together_min_;
-      
     config_->add_notifier(new ConfigNotifier(this));
 
     config_->set_attached(true);
@@ -525,7 +556,8 @@ namespace aspeller {
 
     typedef Vector<SpellerDict *> AllWS; AllWS all_ws;
     for (SpellerDict * i = dicts_; i; i = i->next) {
-      if (i->dict->basic_type == Dict::basic_dict) {
+      if (i->dict->basic_type == Dict::basic_dict ||
+          i->dict->basic_type == Dict::replacement_dict) {
         all_ws.push_back(i);
       }
     }
@@ -551,29 +583,34 @@ namespace aspeller {
 
       ti = &typeid(*cur->dict);
 
-      WSInfo inf;
-      inf.dict = cur->dict;
-      inf.set(*cur);
-
       if (cur->use_to_check) {
-        check_ws.push_back(inf);
-        if (inf.dict->affix_compressed) affix_ws.push_back(inf);
+        check_ws.push_back(cur->dict);
+        if (cur->dict->affix_compressed) affix_ws.push_back(cur->dict);
       }
       if (cur->use_to_suggest) {
-        suggest_ws.push_back(inf);
-        if (inf.dict->affix_compressed) suggest_affix_ws.push_back(inf);
+        suggest_ws.push_back(cur->dict);
+        if (cur->dict->affix_compressed) suggest_affix_ws.push_back(cur->dict);
       }
     }
-    fast_scan   = suggest_ws.front().dict->fast_scan;
-    fast_lookup = suggest_ws.front().dict->fast_lookup;
+    fast_scan   = suggest_ws.front()->fast_scan;
+    fast_lookup = suggest_ws.front()->fast_lookup;
+    have_soundslike = lang_->have_soundslike();
+    have_repl = lang_->have_repl();
+    invisible_soundslike = suggest_ws.front()->invisible_soundslike;
+    soundslike_root_only = suggest_ws.front()->soundslike_root_only;
     affix_compress = !affix_ws.empty();
 
     //
     // Setup suggest
     //
 
-    suggest_.reset(new_default_suggest(this));
-    intr_suggest_.reset(new_default_suggest(this));
+    PosibErr<Suggest *> pe;
+    pe = new_default_suggest(this);
+    if (pe.has_err()) return pe;
+    suggest_.reset(pe.data);
+    pe = new_default_suggest(this);
+    if (pe.has_err()) return pe;
+    intr_suggest_.reset(pe.data);
 
     return no_err;
   }
@@ -614,8 +651,8 @@ namespace aspeller {
   //
   //
 
-  SpellerDict::SpellerDict(LocalDict & d) 
-    : LocalDict(d), special_id(none_id), next(0) 
+  SpellerDict::SpellerDict(Dict * d) 
+    : dict(d), special_id(none_id), next(0) 
   {
     switch (dict->basic_type) {
     case Dict::basic_dict:
@@ -633,30 +670,29 @@ namespace aspeller {
     save_on_saveall = false;
   }
 
-  SpellerDict::SpellerDict(Dict * w, const Language * l, const Config & c, SpecialId id)
+  SpellerDict::SpellerDict(Dict * w, const Config & c, SpecialId id)
     : next(0) 
   {
-    set(l, c);
     dict = w;
     special_id = id;
     switch (id) {
     case main_id:
       if (dict->basic_type == Dict::basic_dict) {
 
-	use_to_check    = true;
-	use_to_suggest  = true;
-	save_on_saveall = false;
+        use_to_check    = true;
+        use_to_suggest  = true;
+        save_on_saveall = false;
 
       } else if (dict->basic_type == Dict::replacement_dict) {
-	
-	use_to_check    = false;
-	use_to_suggest  = false;
-	save_on_saveall = false;
-	
+        
+        use_to_check    = false;
+        use_to_suggest  = false;
+        save_on_saveall = false;
+        
       } else {
-	
-	abort();
-	
+        
+        abort();
+        
       }
       break;
     case personal_id:
