@@ -12,17 +12,10 @@
    CURSES_INCLUDE_WORKAROUND_1
    CURSES_ONLY
    HAVE_GETCH
+   HAVE_MBLEN
    All these macros need to have a true value and not just be defined
 */
 
-//
-// NOTE: The best way to debug this file when the behavior of "aspell
-//       check" differs from the behavior of the orignal Aspell is to
-//       compare against terminos.cc in the old Aspell and look for
-//       differences.  If everthing looks the same then it might be 
-//       a bug in CheckerString.
-//
-
 #include "settings.h"
 
 #ifdef CURSES_NON_POSIX
@@ -33,12 +26,18 @@
 
 #include "asc_ctype.hpp"
 #include "check_funs.hpp"
+#include "convert.hpp"
+#include "config.hpp"
 
+using namespace acommon;
 
 StackPtr<CheckerString> state;
 const char * last_prompt = 0;
 StackPtr<Choices> word_choices;
 StackPtr<Choices> menu_choices;
+Conv null_conv;
+extern Conv dconv;
+extern Conv uiconv;
 
 #if   POSIX_TERMIOS
 
@@ -57,7 +56,7 @@ extern "C" {int getch();}
 
 #if HAVE_LIBCURSES
 
-#include <curses.h>
+#include CURSES_HEADER
 
 #if CURSES_INCLUDE_STANDARD
 
@@ -98,7 +97,13 @@ static SCREEN * term;
 
 #endif // HAVE_LIBCURSES
 
-void cleanup (void) {
+#ifdef HAVE_MBLEN
+#  include <wchar.h>
+#else
+#  define mblen(x,y) (*(x) ? 1 : 0)
+#endif
+
+static void cleanup (void) {
 #if   HAVE_LIBCURSES
   if (use_curses) {
     endwin();
@@ -113,9 +118,9 @@ void cleanup (void) {
 
 #if   HAVE_LIBCURSES
 
-void do_nothing(int) {}
+//static void do_nothing(int) {}
 
-void layout_screen();
+static void layout_screen();
 
 #ifdef CURSES_ONLY
 
@@ -123,7 +128,7 @@ inline void handle_last_signal() {}
 
 #else
 
-void save_state() {
+static void save_state() {
   getyx(choice_w,cur_y,cur_x);
   choice_text_size = COLS;
   choice_text = new char[choice_text_size];
@@ -133,7 +138,7 @@ void save_state() {
   endwin();
 }
 
-void restore_state() {
+static void restore_state() {
   delscreen(term);
   term = newterm(0,stdout,stdin);
   set_term(term);
@@ -157,35 +162,35 @@ void restore_state() {
 }
 
 
-void suspend_handler(int) {
+static void suspend_handler(int) {
   last_signal = SIGTSTP;
 }
   
-void continue_handler(int) {
+static void continue_handler(int) {
   restore_state();
   signal(SIGTSTP, suspend_handler);
   signal(SIGCONT,  continue_handler),
     last_signal = 0;
 }
 
-void resize_handler(int ) {
+static void resize_handler(int ) {
   last_signal = SIGWINCH;
 }
   
-void resize() {
+static void resize() {
   save_state();
   restore_state();
   last_signal = 0;
 }
 
-void suspend() {
+static void suspend() {
   save_state();
   signal(SIGTSTP, SIG_DFL);
   raise(SIGTSTP);
   last_signal = 0;
 }
 
-inline void handle_last_signal() {
+static inline void handle_last_signal() {
   switch (last_signal) {
   case SIGWINCH:
     resize();
@@ -199,7 +204,7 @@ inline void handle_last_signal() {
 
 #endif
 
-void layout_screen() {
+static void layout_screen() {
   text_w = 0;
   menu_w = 0;
   choice_w = 0;
@@ -230,7 +235,12 @@ void layout_screen() {
 }
 
 #endif
+
 void begin_check() {
+  
+  //
+  //
+  //
 #if   HAVE_LIBCURSES
 #if   CURSES_ONLY
   use_curses=true;
@@ -268,7 +278,7 @@ void begin_check() {
 #endif
 #if   POSIX_TERMIOS || (HAVE_LIBCURSES && !CURSES_ONLY)
   if (!isatty (STDIN_FILENO)) {
-    puts("Error: Stdin not a terminal.");
+    puts(_("Error: Stdin not a terminal."));
     exit (-1);
   }
   
@@ -293,9 +303,19 @@ void begin_check() {
 #endif
 }
 
+#ifdef HAVE_WIDE_CURSES
+#  define IS_KEY(k) (res == KEY_CODE_YES && c == KEY_##k)
+#  define NOT_KEY (res != KEY_CODE_YES)
+#else
+#  define IS_KEY(k) (c == KEY_##k)
+#  define NOT_KEY (c < 256)
+#endif
+
 void get_line(String & line) {
 #if   HAVE_LIBCURSES
   if (use_curses) {
+    // FIXME: This won't work correctly when combing characters are
+    // involved.
     menu_text = ReplMenu;
     display_menu();
     wnoutrefresh(choice_w);
@@ -309,7 +329,13 @@ void get_line(String & line) {
     int end_x = begin_x;
     while (true) {
       handle_last_signal();
+#ifdef HAVE_WIDE_CURSES
+      wint_t wi = 0;
+      int res = wget_wch(choice_w, &wi);
+      c = wi;
+#else
       c = wgetch(choice_w);
+#endif
       if (c == ERR) continue;
       if (c == '\r' || c == '\n' || c == KEY_ENTER) 
 	break;
@@ -319,35 +345,61 @@ void get_line(String & line) {
       }
       int y,x;
       getyx(choice_w,y,x);
-      if ((c == KEY_LEFT || c == control('b')) && begin_x < x) {
+      if ((IS_KEY(LEFT) || c == control('b')) && begin_x < x) {
 	wmove(choice_w, y,x-1);
-      } else if ((c == KEY_RIGHT || c == control('f')) && x < end_x) {
+      } else if ((IS_KEY(RIGHT) || c == control('f')) && x < end_x) {
 	wmove(choice_w, y,x+1);
-      } else if (c == KEY_HOME || c == control('a')) {
+      } else if (IS_KEY(HOME) || c == control('a')) {
 	wmove(choice_w, y, begin_x);
-      } else if (c == KEY_END  || c == control('e')) {
+      } else if (IS_KEY(END)  || c == control('e')) {
 	wmove(choice_w, y, end_x);
-      } else if ((c == KEY_BACKSPACE || c == control('h') || c == '\x7F') 
+      } else if ((IS_KEY(BACKSPACE) || c == control('h') || c == '\x7F') 
 		 && begin_x < x) {
 	wmove(choice_w, y,x-1);
 	wdelch(choice_w);
 	--end_x;
-      } else if (c == KEY_DC || c == control('d')) {
+      } else if (IS_KEY(DC) || c == control('d')) {
 	wdelch(choice_w);
 	--end_x;
-      } else if (c == KEY_EOL || c == control('k')) {
+      } else if (IS_KEY(EOL) || c == control('k')) {
 	wclrtoeol(choice_w);
 	end_x = x;
-      } else if (x < max_x && 32 <= c && c != '\x7F' && c < 256) {
+      } else if (x < max_x && 32 <= c && c != '\x7F' && NOT_KEY /*c < 256*/) {
+#ifdef HAVE_WIDE_CURSES
+        wchar_t wc = c;
+        wins_nwstr(choice_w, &wc, 1);
+#else
 	winsch(choice_w, c);
+#endif
 	wmove(choice_w, y, x+1);
 	++end_x;
       }
       wrefresh(choice_w);
     }
+#ifdef HAVE_WIDE_CURSES
+    Vector<wchar_t> wstr;
+    for (int i = begin_x; i < end_x; ++i) {
+      cchar_t cc;
+      attr_t att;
+      short cp;
+      mvwin_wch(choice_w, 0, i, &cc);
+      size_t s = getcchar(&cc, 0, &att, &cp, 0);
+      wstr.resize(s);
+      getcchar(&cc, wstr.data(), &att, &cp, 0);
+      s = wcstombs(0, wstr.data(), 0);
+      if (s != (size_t)-1) {
+        int pos = line.alloc(s); // this always leaves space for the
+                                 // null character
+        wcstombs(line.data(pos), wstr.data(), s);
+      } else {
+        line += '?';
+      }
+    }
+#else
     for (int i = begin_x; i < end_x; ++i) {
       line += mvwinch(choice_w, 0, i) & A_CHARTEXT;
     }
+#endif
     menu_text = StdMenu;
     display_menu();
     doupdate();
@@ -365,7 +417,13 @@ void get_line(String & line) {
     tcsetattr (STDIN_FILENO, TCSANOW, &new_attributes);
 #endif
   }
+  if (uiconv.conv) {
+    line = uiconv(line);
+  }
 }
+
+#undef IS_KEY
+#undef NOT_KEY
     
 void get_choice(int & c) {
 #if   HAVE_LIBCURSES
@@ -407,7 +465,7 @@ void get_choice(int & c) {
 }
 
 #if   HAVE_LIBCURSES
-void new_line(int & l, int y, int height) {
+static void new_line(int & l, int y, int height) {
   --l;
   if (y == height - 1) {
     wmove(text_w, 0, 0);
@@ -417,7 +475,7 @@ void new_line(int & l, int y, int height) {
     wmove(text_w,y+1,0);
   }
 }
-void new_line(int & l, int height) {
+static void new_line(int & l, int height) {
   int y,x;
   getyx(text_w,y,x);
   new_line(l,y,height);
@@ -426,8 +484,9 @@ void new_line(int & l, int height) {
 
 void display_misspelled_word() {
 
-  CheckerString::Iterator word_begin = state->word_begin();
-  CheckerString::Iterator word_end   = state->word_end();
+  const char * word_begin = state->word_begin();
+  const char * word_end   = state->word_end();
+  LineIterator cur_line = state->cur_line();
 
 #if   HAVE_LIBCURSES
 
@@ -437,107 +496,126 @@ void display_misspelled_word() {
     getmaxyx(text_w,height,width);
     assert(height > 0 && width > 0);
 
-    CheckerString::Iterator i = word_begin;
-      
+    LineIterator i = cur_line;
+    
     //
     // backup height/3 lines
     //
     int l = height/3;
-    while (!i.off_end()) {
-      if (*i == '\n') {
-	--l;
-	if (l == 0)
-	  break;
-      }
+    while (!i.off_end() && l > 0) {
+      --l;
       --i;
     }
-    ++i;
-      
-    int last_space_pos = 0;
-    CheckerString::Iterator last_space = i;
-      
+    if (i.off_end()) ++i;
+
     while (l != 0)
       new_line(l,height);
 
-    int y, x;
     l = -1;
-    int attr = A_NORMAL;
-      
+
+    const char * j = i->begin();
     while (!i.off_end()) {
-      getyx(text_w,y,x);
-      if (x == width-1 || *i == '\n') {
-	if (*i != '\n') {
-	  if (dist(last_space, i) < width/3) {
-	    wmove(text_w, y, last_space_pos);
-	    wclrtoeol(text_w);
-	    i = last_space;
-	    ++i;
-	  } 
-	  wmove(text_w, y, width-1);
-	  waddch(text_w,'\\');
-	} 
-	last_space = i;
-	last_space_pos = 0;
-	if (l == 0) break;
-	new_line(l,y,height);
+
+      int y,x;
+      int y0,x0;
+      getyx(text_w, y, x);
+      x0 = x;
+      y0 = y;
+
+      wattrset(text_w,A_NORMAL);
+      int last_space_pos = 0;
+      const char * last_space = i->begin();
+
+      // NOTE: Combining characters after a character at the very end of
+      //   a line will cause an unnecessary word wrap.  Unfortunately I
+      //   do not know how to avoid it as they is no portable way to
+      //   find out if the next character will combine with the
+      //   previous.
+      while (j < i->end() && x0 <= x && *j != '\n')
+      {
+        if (asc_isspace(*j)) {
+          last_space_pos = x;
+          last_space = j;
+        }
+        if (j == word_begin) {
+          wattrset(text_w,A_REVERSE);
+          l = height*2/3;
+        } else if (j == word_end) {
+          wattrset(text_w,A_NORMAL);
+        }
+        int len = mblen(j, MB_CUR_MAX);
+        if (len > 0) {
+          waddnstr(text_w, const_cast<char *>(j), len);
+        } else {
+          waddch(text_w, ' ');
+          len = 1;
+        }
+        j += len;
+        x0 = x;
+        getyx(text_w, y, x);
+      }
+      y = y0;
+      if (j == i->end() || *j == '\n') {
+        ++i;
+        j = i->begin();
       } else {
-	if (asc_isspace(*i)) {
-	  getyx(text_w,y,last_space_pos);
-	  last_space = i;
-	}
+        if (x - last_space_pos < width/3) {
+          wmove(text_w, y, last_space_pos);
+          wclrtoeol(text_w);
+          j = last_space + 1;
+        }
+        wmove(text_w, y, width-1);
+        wattrset(text_w,A_NORMAL);
+        waddch(text_w,'\\');
       }
-      if (i == word_begin) {
-	attr = A_REVERSE;
-	l = height*2/3;
-      } else if (i == word_end) {
-	attr = A_NORMAL;
-      }
-      if (*i != '\n')
-	waddch(text_w, (unsigned char)*i | attr);
-      ++i;
+
+      if (l == 0) break;
+      new_line(l,y,height);
     }
 
-    while (l != 0) {
+    while (l != 0)
       new_line(l,height);
-    }
       
     wnoutrefresh(text_w);
+
   } else if (use_curses && !text_w) {
     // do nothing
   } else
 #endif
   {
-    CheckerString::Iterator  i, line_begin, line_end;
-    for (line_begin = word_begin; 
-	 !line_begin.off_end() && *line_begin != '\n';
-	 --line_begin);
-    ++line_begin;
-    for (line_end = word_end;
-	 !line_end.off_end() && *line_end != '\n';
-	 ++line_end);
-    for(i = line_begin; i != word_begin; ++i)
-      putchar(*i);
+    int pre  = word_begin - cur_line->begin();
+    int post = cur_line->end() - word_end;
+    if (pre)
+      fwrite(cur_line->begin(), pre, 1, stdout);
     putchar('*');
-    for(; i != word_end; ++i)
-      putchar(*i);
+    fwrite(word_begin, word_end - word_begin, 1, stdout);
     putchar('*');
-    for(; i != line_end; ++i) 
-      putchar(*i);
-    putchar('\n');
+    if (post)
+      fwrite(word_end, post, 1, stdout);
   }
-
 }
 
-template <class O>
-static void print_truncate(O *out, const char * word, int width) {
+static inline void put (FILE * out, char c) {putc(c, out);}
+static inline void put (FILE * out, const char * s) {fputs(s, out);}
+static inline void put (FILE * out, const char * s, int size) {fwrite(s, size, 1, out);}
+
+static void print_truncate(FILE * out, const char * word, int width) {
   int i;
-  for (i = 0; i < width-1 && word[i]; ++i)
-    put(out,word[i]);
+  int len = 0;
+  for (i = 0; i < width-1 && *word; word += len, ++i) {
+    len = mblen(word, MB_CUR_MAX);
+    if (len > 0) {
+      put(out, word, len);
+    } else {
+      put(out, ' ');
+      len = 1;
+    }
+  }
   if (i == width-1) {
-    if (word[i] == '\0')
+    if (word == '\0')
       put(out,' ');
-    else if (word[i+1] == '\0')
-      put(out,word[i]);
+    else if (word[len] == '\0')
+      put(out, word, len);
     else
       put(out,'$');
     ++i;
@@ -546,30 +624,27 @@ static void print_truncate(O *out, const char * word, int width) {
     put(out,' ');
 }
 
-template <class O>
-static void display_menu(O * out, const Choices * choices, int width) {
+static void display_menu(FILE * out, const Choices * choices, int width, 
+                         Conv & conv = null_conv) 
+{
   if (width <= 11) return;
   Choices::const_iterator i = choices->begin();
   while (i != choices->end()) {
     put(out,i->choice);
     put(out,") ");
-    print_truncate(out, i->desc, width/2 - 4);
+    print_truncate(out, conv(i->desc), width/2 - 4);
     put(out,' ');
     ++i;
     if (i != choices->end()) {
       put(out,i->choice);
       put(out,") ");
-      print_truncate(out, i->desc, width/2 - 4);
+      print_truncate(out, conv(i->desc), width/2 - 4);
       ++i;
     }
-    new_line(out);
+    putc('\n', out);
   }
 }
 
-static inline void put (FILE * out, char c) {putc(c, out);}
-static inline void put (FILE * out, const char * s) {fputs(s, out);}
-static inline void new_line(FILE * out) {putc('\n', out);}
-
 #if   HAVE_LIBCURSES
 
 static inline void put (WINDOW * w, char c) 
@@ -580,15 +655,61 @@ static inline void put (WINDOW * w, const char * c)
 {
   waddstr(w,const_cast<char *>(c));
 }
-static inline void new_line(WINDOW * w) 
+static inline void put (WINDOW * w, const char * c, int size)
 {
+  waddnstr(w,const_cast<char *>(c), size);
+}
+
+static void print_truncate(WINDOW * out, const char * word, int width) {
+  int len = 0;
   int y,x;
-  getyx(w,y,x);
-  wmove(w,y+1,0);
-} 
+  int y0;
+  getyx(out, y, x);
+  int stop = x + width - 1;
+  while (x <= stop && *word) {
+    len = mblen(word, MB_CUR_MAX);
+    if (len <= 0) len = 1;
+    put(out, word, len);
+    word += len;
+    y0 = y;
+    getyx(out, y, x);
+    if (y != y0) {y = y0; x = stop + 1; break;}
+  }
+  for (; x <= stop; ++x)
+    put(out, ' ');
+  if (x > stop && *word) {
+    wmove(out, y, stop);
+    put(out,'$');
+  }
+}
+
+static void display_menu(WINDOW * out, const Choices * choices, int width,
+                         Conv & conv = null_conv) 
+{
+  if (width <= 11) return;
+  Choices::const_iterator i = choices->begin();
+  int y,x;
+  getyx(out, y, x);
+  while (i != choices->end()) {
+    wclrtoeol(out);
+    put(out,i->choice);
+    put(out,") ");
+    print_truncate(out, conv(i->desc), width/2 - 4);
+    ++i;
+    if (i != choices->end()) {
+      wmove(out, y, width/2);
+      put(out,i->choice);
+      put(out,") ");
+      print_truncate(out, conv(i->desc), width/2 - 4);
+      ++i;
+    }
+    ++y;
+    wmove(out, y, 0);
+  }
+}
 
 #endif
-  
+
 void display_menu() {
 #if   HAVE_LIBCURSES
   if (use_curses && menu_w) {
@@ -598,7 +719,7 @@ void display_menu() {
       getmaxyx(menu_w,height,width);
       werase(menu_w);
       wmove(menu_w,0,0);
-      display_menu(menu_w, word_choices.get(), width);
+      display_menu(menu_w, word_choices.get(), width, dconv);
       wmove(menu_w,5,0);
       display_menu(menu_w, menu_choices.get(), width);
       wnoutrefresh(menu_w);
@@ -613,35 +734,87 @@ void display_menu() {
 	const char * desc;
       };
       static MenuLine menu_items[9] = {
-	{0, "Enter", "", "Accept Changes"} 
-	, {0, "Backspace", "Control-H", "Delete the previous character"}
-	, {"kcub1", "Left", "Control-B", "Move Back 1 space"}
-	, {"kcuf1", "Right", "Control-F", "Move Forward 1 space"}
-	, {"khome", "Home", "Control-A", "Move to the beginning of the line"}
-	, {"kend" , "End",  "Control-E", "Move to the end of the line"}
-	, {"kdch1", "Delete", "Control-D", "Delete the next charcter"}
-	, {0, "", "Control-K", "Kill all characters to the EOL"}
-	, {0, "", "Control-C", "Abort This Operation"}
+	{0, 
+         /* TRANSLATORS: This is a literal Key.*/
+         N_("Enter"), 
+         "", 
+         N_("Accept Changes")},
+        {0,
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Backspace"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-H"), 
+         N_("Delete the previous character")},
+	{"kcub1", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Left"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-B"), 
+         N_("Move Back one space")},
+	{"kcuf1", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Right"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-F"), 
+         N_("Move Forward one space")},
+	{"khome", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Home"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-A"), 
+         N_("Move to the beginning of the line")},
+	{"kend" , 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("End"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-E"), 
+         N_("Move to the end of the line")},
+	{"kdch1", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Delete"), 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-D"), 
+         N_("Delete the next character")},
+	{0, 
+         "", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-K"), 
+         N_("Kill all characters to the EOL")},
+	{0, 
+         "", 
+         /* TRANSLATORS: This is a literal Key. */
+         N_("Control-C"), 
+         N_("Abort This Operation")},
       };
       scrollok(menu_w,false);
       werase(menu_w);
+      int beg = 0,end = 0;
+      int y,x,x0;
       for (int i = 0; i != 9; ++i) {
-	wmove(menu_w, i, 0);
-	int w = width;
-	int fun_key_desc_width = 12;
-	int control_key_desc_width = 12;
-	if (w < fun_key_desc_width) fun_key_desc_width = w;
+        wmove(menu_w, i, beg);
 	if (menu_items[i].capname == 0 
-	    || tigetstr(const_cast<char *>(menu_items[i].capname)) != 0) 
-	  print_truncate(menu_w, menu_items[i].fun_key, fun_key_desc_width);
-	else
-	  print_truncate(menu_w, "", fun_key_desc_width);
-	w -= fun_key_desc_width;
-	if (w < control_key_desc_width) control_key_desc_width = w;
-	print_truncate(menu_w, menu_items[i].control_key, 
-		       control_key_desc_width);
-	w -= control_key_desc_width;
-	print_truncate(menu_w, menu_items[i].desc, w);
+            || tigetstr(const_cast<char *>(menu_items[i].capname)) != 0)
+        {
+          getyx(menu_w, y, x0);
+          put(menu_w, gt_(menu_items[i].fun_key));
+          getyx(menu_w, y, x);
+          if (x < beg || y != i) end = width;
+          else if (x > end) end = x;
+        }
+      }
+      beg = end + 2;
+      if (beg < width) for (int i = 0; i != 9; ++i) {
+        wmove(menu_w, i, beg);
+        put(menu_w, gt_(menu_items[i].control_key));
+        getyx(menu_w, y, x);
+        if (x < beg || y != i) end = width;
+        else if (x > end) end = x;
+      }
+      beg = end + 2;
+      int w = width - beg;
+      if (w > 1) for (int i = 0; i != 9; ++i) {
+        wmove(menu_w, i, beg);
+        print_truncate(menu_w, gt_(menu_items[i].desc), w);
       }
       wnoutrefresh(menu_w);
     }
@@ -650,7 +823,7 @@ void display_menu() {
   } else 
 #endif
   {
-    display_menu(stdout, word_choices.get(), 80);
+    display_menu(stdout, word_choices.get(), 80, dconv);
     display_menu(stdout, menu_choices.get(), 80);
   }
 }
